import axios from 'axios'

// connection config
let url

export default class Database {

  // create collection
  constructor() {
    this.collectionName = ''
  }

  // set collection
  collection(collectionName) {
    this.collectionName = collectionName
    return this
  }

  // count documents in collection
  async count(query, options) {
    if (!this.collectionName) throw 'No collection specified'
    let response = await axios.post(`${url}/collection/${this.collectionName}/count`, {query, options})
    return response.data.count
  }

  // delete many from a collection
  async deleteMany(query, options) {
    if (!this.collectionName) throw 'No collection specified'
    let response = await axios.post(`${url}/collection/${this.collectionName}/deletemany`, {query, options})
    return response.data.result
    return await this.exec('deleteMany', query, null, options)
  }

  // delete one from a collection
  async deleteOne(query, options) {
    if (!this.collectionName) throw 'No collection specified'
    let response = await axios.post(`${url}/collection/${this.collectionName}/deleteone`, {query, options})
    return response.data.result
  }

  // drop collection
  async drop() {
    if (!this.collectionName) throw 'No collection specified'
    let response = await axios.post(`${url}/collection/${this.collectionName}/drop`)
    return response.data.count
  }

  // find from collection
  async findMany(query, options) {
    if (!this.collectionName) throw 'No collection specified'
    let response = await axios.post(`${url}/collection/${this.collectionName}/findmany`, {query, options})
    return response.data.docs
  }

  // find one from a collection
  async findOne(query, options) {
    if (!this.collectionName) throw 'No collection specified'
    let response = await axios.post(`${url}/collection/${this.collectionName}/findone`, {query, options})
    return response.data.doc
  }

  // insert documents into a collection
  async insertMany(docs, options) {
    if (!this.collectionName) throw 'No collection specified'
    let response = await axios.post(`${url}/collection/${this.collectionName}/insertmany`, {docs, options})
    return docs.map((doc, index) => Object.assign({_id: response.data.insertedIds[index]}, doc))
  }

  // insert a document into a collection
  async insertOne(doc, options) {
    if (!this.collectionName) throw 'No collection specified'
    let response = await axios.post(`${url}/collection/${this.collectionName}/insertone`, {doc, options})
    doc._id = response.data.insertedId
    return doc
  }

  static async listCollections() {
    let response = await axios.post(`${url}/listcollections`)
    return response.data.collections
  }

  // replace a document in a collection
  async replaceOne(query, doc, options) {
    if (!this.collectionName) throw 'No collection specified'
    let response = await axios.post(`${url}/collection/${this.collectionName}/replaceone`, {query, doc, options})
    return response.data.result
  }

  // update a document in a collection
  async updateMany(query, doc, options) {
    if (!this.collectionName) throw 'No collection specified'
    let response = await axios.post(`${url}/collection/${this.collectionName}/updatemany`, {query, doc, options})
    return response.data.result
  }

  // update a document in a collection
  async updateOne(query, doc, options) {
    if (!this.collectionName) throw 'No collection specified'
    let response = await axios.post(`${url}/collection/${this.collectionName}/updateone`, {query, doc, options})
    return response.data.result
  }

  // static alias
  static collection(...args) { return new this().collection(...args) }

  // set config
  static setUrl(newUrl) { url = newUrl }

}
